#' Random effects model with measurement error likelihood
#'
#' Returns minus log likelihood of the random effects model with measurement
#' error from Korn et al. (2005). This is an implementation of equation 7 from
#' that paper.
#'
#' @param Yeff Vector of estimates of the effect of treatment on Y, the
#'   definitive outcome
#' @param Xeff Vector of estimates of the effect of treatment on X, the
#'   surrogate outcome
#' @param Yse Vector of standard errors of the estimates on Y
#' @param Xse Vector of standard errors of the estimates on X
#' @param param Vector of length 5 of parameters
#'
#' @return A vector of length 1, the minus log likelihood value
#'
#' @references Korn, EL, Albert, PS, and McShane LM (2005). Assessing surrogates
#'   as trial endpoints using mixed models. Statistics in Medicine; 24:163--182.
#'
#' @export

like.hood.model4 <- function(Yeff, Xeff, Yse, Xse, param){

  stopifnot(length(param) == 5)
  stopifnot(length(Xeff) == length(Yeff))
  stopifnot(length(Yse) == length(Yeff))
  stopifnot(length(Xse) == length(Xeff))

  vmg <- param[5]
  beta <- param[3]
  muy <- param[2]
  vmx <- param[4]
  mux <- param[1]
  n <- length(Yeff)

  c11 <-  vmx + Xse^2
  c22 <- beta^2 * vmx + vmg + Yse^2
  c12 <-  beta * vmx


  sigma <- rbind(cbind(diag(c11), diag(c12, nrow = n)),
                 cbind(diag(c12, nrow = n), diag(c22)))

  XX <- c(Xeff, Yeff)
  mu = c(rep(mux, n), rep(muy + beta * mux, n))

  hood <- mvtnorm::dmvnorm(XX, mean = mu, sigma = sigma, log = TRUE)

  -hood


}


#' Estimates the random effects model
#'
#' Fits the random effects model with measurement
#' error from Korn et al. (2005). This is an implementation of equation 7 from
#' that paper. A finite-sample variance adjustment is incorporated for the variance components.
#'
#' @param Yeff Vector of estimates of the effect of treatment on Y, the
#'   definitive outcome
#' @param Xeff Vector of estimates of the effect of treatment on X, the
#'   surrogate outcome
#' @param Yse Vector of standard errors of the estimates on Y
#' @param Xse Vector of standard errors of the estimates on X
#' @param start Vector of length 5 of starting values
#'
#' @return A list with same component as \link{optim}, with the additional elements:
#' \describe{
#'  \item{n}{Sample size}
#'  \item{std.err}{Standard error estimates}
#'  \item{summary}{Summary table with estimates and their standard errors}
#' }
#'
#' @references Korn, EL, Albert, PS, and McShane LM (2005). Assessing surrogates
#'   as trial endpoints using mixed models. Statistics in Medicine; 24:163--182.
#'
#' @export


fit_model4 <- function(Yeff, Xeff, Yse, Xse, start = c(.4,-.2, 0.05, .1, .005)) {

  stopifnot(length(start) == 5)
  stopifnot(length(Xeff) == length(Yeff))
  stopifnot(length(Yse) == length(Yeff))
  stopifnot(length(Xse) == length(Xeff))

    fit4 <- optim(start, function(par) like.hood.model4(Yeff, Xeff, Yse, Xse, par),
                  method = "L-BFGS-B", lower = c(-Inf, -Inf, -Inf, 0, 0), upper = c(Inf, Inf, Inf, Inf, Inf),
                  control = list(maxit = 1000), hessian = TRUE)

    stopifnot(fit4$convergence == 0)

    fit4$std.err <- sqrt(diag(solve(fit4$hessian)))

    fit4$n <- length(Yeff)
    names(fit4$par) <- names(fit4$std.err) <- c("mux", "muy", "beta", "vmx", "vmg")
    fit4$par["vmx"] <- fit4$par["vmx"] * fit4$n/(fit4$n - 1)
    fit4$par["vmg"] <- fit4$par["vmg"] * fit4$n/(fit4$n - 3)

    fit4$std.err["vmx"] <- fit4$std.err["vmx"] * fit4$n/(fit4$n - 1)
    fit4$std.err["vmg"] <- fit4$std.err["vmg"] * fit4$n/(fit4$n - 3)


    summ <- round(cbind(fit4$par, fit4$std.err), 4)
    colnames(summ) <- c("Est", "SE")
    fit4$summary <- summ

    fit4

}
