#' Generate data according to model 4
#'
#' Using the observed standard errors of the 10 trials, this generates data a new set of trials, recycling as necessary
#'
#' @param n Sample size
#' @param vmg variance of g
#' @param vmx variance of x
#' @param beta Regression coefficient for association between treatment effect on surrogate and treatment effect on definitive outcome
#'
#' @return A data frame
#'
#' @export

gen_dataset <- function(n = 11, vmg = 0.0075, vmx = .1119, beta = 0.0452){

  # recycle
  dex <- 1:n
  dex[dex > nrow(trials10)] <-  (dex[dex > nrow(trials10)] %% nrow(trials10)) + 1

  sigma.pcr <- trials10$PCR.SE[dex]
  sigma.hr <- trials10$DFS.SE[dex]
  ei <- rnorm(n, mean = 0, sd = sigma.pcr)
  deltai <- rnorm(n, mean = 0, sd = sigma.hr)
  gi <- rnorm(n, mean = 0, sd = sqrt(2) * sqrt(vmg))
  mi <- rnorm(n, mean = 0, sd = sqrt(2) * sqrt(vmx))

  Xi <- .38 + mi + ei
  Yi <- -.18 + beta * mi + gi + deltai

  data.frame(Xi, sigma.pcr, Yi, sigma.hr, Yi.star = -.18 + beta * mi + gi)

}

#' Assess prediction interval coverage
#'
#' Check for coverage of confidence and prediction intervals for the treatment effect on the definitive outcome in a new trial
#'
#' @param i Index, not used
#' @param n sample size
#' @param zst Z or T score that determines coverage probability of intervals, typically 95\%
#' @param vmg variance of g
#' @param vmx variance of x
#'
#' @return logical vector of length 2, true value in confidence interval/prediction interval
#' @export


run_pred_coverage <- function(i, n = 10, zst = 1.96, vmg = 0.0075, vmx = .1119){

  withpred <- gen_dataset(n = n, vmg = vmg, vmx = vmx)
  tes <- withpred[1:n, ]

  gi <- rnorm(1, mean = 0, sd = sqrt(2) * sqrt(vmg))
  mi <- rnorm(1, mean = 0, sd = sqrt(2) * sqrt(vmx))

  Xi <- .4
  Yi <- -.18 + 0.0452 * (mi + .38) + gi

  topred <- data.frame(Xi = Xi, Yi = Yi, sigma.pcr = .2, sigma.hr = .11)


  start <- c(.4,-.2, 0.05, .1, .005)

  fit4 <- fit_model4(tes$Yi, tes$Xi, tes$sigma.hr, tes$sigma.pcr, start)

  checkrow <- predict_klmfit(fit4, topred$Xi, topred$sigma.pcr^2)
  lim <- checkrow$delta + c(-zst, zst) * checkrow$se.pred
  newva <- topred$Yi > lim[1] & topred$Yi < lim[2]

  ### coverage for predicted value

  lim2 <- checkrow$delta + c(-zst, zst) * checkrow$se.delta
  truf <- topred$Xi * 0.0452 * (vmx / (vmx + .2^2)) + 0.0452 * 0.38 * (1 - (vmx / (vmx + .2^2))) - 0.18
  pred <- truf > lim2[1] & truf < lim2[2]

  ret <- c(newva, pred)
  names(ret) <- c("new value", "prediction")
  ret


}